<?php
/*
Plugin Name: BDN duplicate images
Author: William P. Davis, Bangor Daily News
Author URI: http://wpdavis.com/
Version: 0.1-beta
License: GPL v2 - http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
*/


// Add the attachment link to the media library page
add_filter( 'attachment_fields_to_edit', 'bdn_add_link_to_attach_image_to_post', 20, 2 );
function bdn_add_link_to_attach_image_to_post( $fields, $attachment ) {

	if( empty( $_GET[ 'post_id' ] ) )
		return $fields;
		
	if( empty( $_GET[ 'tab' ] ) || $_GET[ 'tab' ] != 'library' )
		return $fields;

	$post_id = (int) $_GET[ 'post_id' ];
	
	if( empty( $post_id ) )
		return $fields;
	
	$nonce = wp_create_nonce( 'bdn-attach_' . $attachment->ID );
	$attach_image_action_url = admin_url( "media-upload.php?tab=library&post_id=$post_id" );

	$fields[ 'bdn_attach' ] = array(
		'label' => __('Attach to story:'),
		'input' => 'html',
		'html' => "<a href='$attach_image_action_url&bdn-attach=$attachment->ID&_attachnonce=$nonce'>Attach</a> "

	);

	return $fields;
	
}

//Perform the duplicating action
add_action( 'admin_init', 'bdn_attach_image_to_post' );
function bdn_attach_image_to_post() {
	global $pagenow;
	
	
	//Check to make sure we're on the right page and performing the right action
	if( 'media-upload.php' != $pagenow )
		return false;

	if( empty( $_GET[ 'post_id' ] ) || empty( $_GET[ 'bdn-attach' ] ) || empty( $_GET[ '_attachnonce' ] ) )
		return false;
		
	if( empty( $_GET[ 'tab' ] ) || $_GET[ 'tab' ] != 'library' )
		return false;

	$post_id = (int) $_GET[ 'post_id' ];
	
	if( empty( $post_id ) )
		return false;

	if( !wp_verify_nonce( $_GET[ '_attachnonce' ], 'bdn-attach_' . $_GET[ 'bdn-attach' ] ) )	
		return false;
		
	$attachment_id = (int) $_GET[ 'bdn-attach' ];
	
	if( empty( $attachment_id ) )
		return false;
		
	//Get the attachment we're working with as an array
	
	$attachment = get_post( $attachment_id, ARRAY_A );

	//If the attachment doesn't have a post parent, simply change it to the attachment we're working with and be done with it
	if( empty( $attachment[ 'post_parent' ] ) ) {
		wp_update_post(
			array(
				'ID' => $attachment[ 'ID' ],
				'post_parent' => $post_id
			)
		);
	} else {
		//Else, unset the attachment ID, change the post parent and insert a new attachment
		unset( $attachment[ 'ID' ] );
		$attachment[ 'post_parent' ] = $post_id;
		$new_attachment_id = wp_insert_post( $attachment );
		
		
		//Now, duplicate all the custom fields. (There's probably a better way to do this)
		$custom_fields = get_post_custom( $attachment_id );

		foreach( $custom_fields as $key => $value ) {
			//The attachment metadata wasn't duplicating correctly so we do that below instead
			if( $key != '_wp_attachment_metadata' )
				update_post_meta( $new_attachment_id, $key, $value[0] );
		}
		
		//Carry over the attachment metadata
		$data = wp_get_attachment_metadata( $attachment_id );
		wp_update_attachment_metadata( $new_attachment_id, $data );
		
		
	}
	
	//Redirect to the gallery for that post and exit
	wp_safe_redirect( admin_url( 'media-upload.php?tab=gallery&post_id=' . $post_id ) );
	exit();

}